//+------------------------------------------------------------------+
//|                                                   BlockHider.mqh |
//+------------------------------------------------------------------+

#property copyright "TheXpert"
#property link      "theforexpert@gmail.com"

#include "Assert.mqh"
#include "CommonUID.mqh"

const ushort BLOCK_HIDER_EVENT = 2;

class BlockHider
{
   public:
      BlockHider();
      ~BlockHider();
      
      string Name() const;
      bool Minimized() const;
      
      void Draw(int x, int y, int chartID, int subWnd, int corner = 0);
      
      void OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam);
   
   private:
      void HideEvent();

      void CheckHider();
      void CreateHider();

      string Caption() const;
      
   private:
      int      m_X;
      int      m_Y;

      string   m_Name;
      int      m_Window;
      int      m_ChartID;
      int      m_Corner;
      
      bool     m_Minimized;
};

BlockHider::BlockHider(void)
{
   m_Name = "BlockHiderObj#" + string(GetUID());
   m_Minimized = false;
}

void BlockHider::OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam)
{
   if (sparam == m_Name)
   {
      if (id == CHARTEVENT_OBJECT_DELETE)
      {
         CreateHider();
      }
      else if (id == CHARTEVENT_OBJECT_CLICK)
      {
         m_Minimized = !m_Minimized;
         ObjectSetString(m_ChartID, m_Name, OBJPROP_TEXT, Caption());
         ObjectSetInteger(m_ChartID, m_Name, OBJPROP_STATE, false);
         HideEvent();
      }
   }
}

string BlockHider::Name() const
{
   return m_Name;
}

void BlockHider::~BlockHider()
{
   ObjectDelete(m_ChartID, m_Name);
}

bool BlockHider::Minimized() const
{
   return m_Minimized;
}

void BlockHider::Draw(int x, int y, int chartID, int subWnd, int corner = 0)
{
   m_X = x;
   m_Y = y;
   
   m_Window = subWnd;
   m_ChartID = chartID;
   m_Corner = corner;

   CheckHider();
   
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_XDISTANCE, m_X);
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_YDISTANCE, m_Y);
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_CORNER, m_Corner);
}

void BlockHider::HideEvent()
{
   EventChartCustom(m_ChartID, BLOCK_HIDER_EVENT, m_Minimized, 0, m_Name);
}

void BlockHider::CheckHider()
{
   int wnd = ObjectFind(m_ChartID, m_Name);
   
   if (wnd >= 0 && wnd != m_Window)
   {
      ObjectDelete(m_ChartID, m_Name);
      CreateHider();
   }
   else if (wnd < 0)
   {
      CreateHider();
   }
}

void BlockHider::CreateHider()
{
   ObjectCreate(m_ChartID, m_Name, OBJ_BUTTON, m_Window, 0, 0);

   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_SELECTABLE, false);
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_XSIZE, 15);
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_YSIZE, 15);
   
   ObjectSetString(m_ChartID, m_Name, OBJPROP_TEXT, Caption());
}

string BlockHider::Caption() const
{
   if (m_Minimized) return "+";
   return "-";
}