//+------------------------------------------------------------------+
//|                                                   BlockMover.mqh |
//+------------------------------------------------------------------+
#property copyright "TheXpert"
#property link      "theforexpert@gmail.com"

#include "Assert.mqh"
#include "CommonUID.mqh"

const ushort BLOCK_MOVER_EVENT = 1;

class BlockMover
{
   public:
      BlockMover();
      ~BlockMover();
      
      int X() const;
      int Y() const;
      string Name() const;
      
      void OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam);
      
      void Init(int x, int y, int subWindow, int chartID = 0, char symbolCode = 118, color clr = White, int corner = 0);
   
   private:
      void CreateMover();
      void OnDelete();
      void OnChange();
      
      void PosChangedEvent();
      
   
      int      m_X;
      int      m_Y;
      int      m_InitialX;
      int      m_InitialY;

      string   m_Name;
      char     m_SymbolCode;
      bool     m_Inited;
      int      m_Window;
      int      m_ChartID;
      color    m_Color;
      int      m_Corner;
};

BlockMover::BlockMover(void)
{
   m_Name = "BlockMoverObj#" + string(GetUID());
   m_Inited = false;
}

void BlockMover::Init(int x, int y, int subWindow, int chartID = 0, char symbolCode = 118, color clr = White, int corner = 0)
{
   m_X = x;
   m_Y = y;
   m_InitialX = x;
   m_InitialY = y;
   
   m_Window = subWindow;
   m_ChartID = chartID;
   m_SymbolCode = symbolCode;
   
   m_Color = clr;
   m_Corner = corner;
   
   CreateMover();
   
   m_Inited = true;
}

void BlockMover::CreateMover()
{
   m_X = m_InitialX;
   m_Y = m_InitialY;

   ObjectDelete(m_ChartID, m_Name);
   ObjectCreate(m_ChartID, m_Name, OBJ_LABEL, m_Window, 0, 0);
   
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_CORNER, m_Corner);
   
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_XDISTANCE, m_InitialX);
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_YDISTANCE, m_InitialY);
   
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_COLOR, m_Color);

   ObjectSetString(m_ChartID, m_Name, OBJPROP_TEXT, CharToString(m_SymbolCode));
   ObjectSetString(m_ChartID, m_Name, OBJPROP_FONT, "Wingdings");

   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_FONTSIZE, 10);
   ObjectSetInteger(m_ChartID, m_Name, OBJPROP_SELECTABLE, true);

   PosChangedEvent();
}

int BlockMover::X() const
{
   Assert(m_Inited, "m_Inited at line "  + string(__LINE__) + " function "  + __FUNCTION__ + " file " + __FILE__);
   return m_X;
}

int BlockMover::Y() const
{
   Assert(m_Inited, "m_Inited at line "  + string(__LINE__) + " function "  + __FUNCTION__ + " file " + __FILE__);
   return m_Y;
}

void BlockMover::OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam)
{
   if (sparam == m_Name)
   {
      switch (id)
      {
         case CHARTEVENT_OBJECT_DRAG: 
               OnChange(); 
            break;
         case CHARTEVENT_OBJECT_CHANGE: 
               OnChange(); 
            break;
         case CHARTEVENT_OBJECT_DELETE: 
               OnDelete(); 
            break;
      }
   }
}

void BlockMover::OnDelete()
{
   CreateMover();
}

void BlockMover::OnChange()
{
   m_X = int(ObjectGetInteger(m_ChartID, m_Name, OBJPROP_XDISTANCE));
   m_Y = int(ObjectGetInteger(m_ChartID, m_Name, OBJPROP_YDISTANCE));
   PosChangedEvent();
}

void BlockMover::PosChangedEvent()
{
   EventChartCustom(m_ChartID, BLOCK_MOVER_EVENT, m_X, double(m_Y), m_Name);
}

string BlockMover::Name() const
{
   return m_Name;
}

void BlockMover::~BlockMover()
{
   ObjectDelete(m_ChartID, m_Name);
}